"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigResolver = void 0;
const events_1 = require("events");
const path_1 = require("path");
const utils_1 = require("./utils");
const fs_1 = require("fs");
const FileSystemExtensions_1 = require("./FileSystemExtensions");
class ConfigResolver extends events_1.EventEmitter {
    constructor(appName, configFile) {
        super();
        this._appName = appName;
        this._configFile = configFile;
    }
    getPaths() {
        const configFile = this._configFile || 'config.json';
        const appName = this._appName;
        const paths = [];
        const xdgConfig = process.env.XDG_CONFIG_HOME ||
            (process.env.HOME && (0, path_1.join)(process.env.HOME, '.config'));
        if (xdgConfig && (0, FileSystemExtensions_1.folderExists)(xdgConfig)) {
            paths.push({
                path: (0, path_1.join)(xdgConfig, appName, configFile),
                type: 'xdg',
            });
        }
        if (process.platform === 'win32' &&
            process.env.APPDATA &&
            (0, FileSystemExtensions_1.folderExists)(process.env.APPDATA)) {
            paths.push({
                path: (0, path_1.resolve)((0, path_1.join)(process.env.APPDATA, appName, configFile)),
                type: 'win',
            });
        }
        paths.push({
            path: (0, path_1.resolve)((0, path_1.join)('.', appName, configFile)),
            type: 'def',
        });
        paths.push({
            path: (0, path_1.resolve)((0, path_1.join)('.', configFile)),
            type: 'old',
        });
        return paths;
    }
    createConfigFile(configPath) {
        (0, utils_1.mkdirpSync)((0, path_1.dirname)(configPath.path));
        const defaultConfig = require.resolve('./example.json');
        const createdConfig = (0, fs_1.readFileSync)(defaultConfig, 'utf8');
        (0, fs_1.writeFileSync)(configPath.path, createdConfig);
        return configPath.path;
    }
    findOrGenerateConfig() {
        let config = this.findConfig();
        if (config)
            return config;
        const paths = this.getPaths();
        const newConfig = this.createConfigFile(paths[0]);
        this.emit('info', `Creating default config file in ${config}`);
        return newConfig;
    }
    findConfig() {
        const paths = this.getPaths();
        const existingConfig = paths.find(o => (0, FileSystemExtensions_1.fileExists)(o.path));
        return existingConfig && existingConfig.path;
    }
    resolveConfig() {
        const path = this.getOrGenerateConfig();
        const config = JSON.parse((0, fs_1.readFileSync)(path, 'utf8'));
        config.self_path = path;
        return config;
    }
    getOrGenerateConfig() {
        if (this._configFile) {
            return (0, path_1.resolve)(this._configFile);
        }
        return this.findOrGenerateConfig();
    }
}
exports.ConfigResolver = ConfigResolver;
//# sourceMappingURL=ConfigResolver.js.map